@@ -1565,11 +1565,12 @@ def get_metrics(X,y):
     if dic is None:
         dic = {}
  
+    ber = 1 # set maximum BER
     try:
-        ber, _ = BER(X,y)
-        dic['classification.BER'] = ber
+        ber, _ = BER(X,y)        
     except:
         print('Exception in GaussianNB.')
+    dic['classification.BER'] = ber
 
     svm_width, _ = SVM_Margin_Width(X, y)
     dic['classification.SVM.Margin'] = svm_width
@@ -1629,19 +1630,29 @@ def get_metrics(X,y):
     dic['test.CHISQ.CHI2'] = C
     dic['test.CHISQ.CHI2.max'] = np.max(C)
 
-    p, H = KW(X,y)
+    H = [scipy.stats.chi2.ppf(.5, len(set(y))-1)]* X.shape[1] # H follows chi2, its critical value of chi2(k-1) at 0.5
+    p = [.5] * X.shape[1]
+    try:
+        p, H = KW(X,y)
+    except Exception as e:
+        print('KW Exception: ', e)
     dic['test.KW'] = p
     dic['test.KW.min'] = np.min(p)
     dic['test.KW.min.log10'] = np.log10 (np.min(p))
     dic['test.KW.H'] = H
     dic['test.KW.H.max'] = np.max(H)
 
-    p, T, _ = MedianTest(X,y)
+    T = [scipy.stats.chi2.ppf(.5, len(set(y))-1)]* X.shape[1] # T follows chi2, its critical value of chi2(k-1) at 0.5
+    p = [.5] * X.shape[1]
+    try:
+        p, T, _ = MedianTest(X,y)
+    except Exception as e:
+        print('MedianTest Exception: ', e)
     dic['test.Median'] = p
     dic['test.Median.min'] = np.min(p)
     dic['test.Median.min.log10'] = np.log10 (np.min(p))
-    dic['test.Median.CH2'] = T
-    dic['test.Median.CH2.max'] = np.max(T)
+    dic['test.Median.CHI2'] = T
+    dic['test.Median.CHI2.max'] = np.max(T)
 
     if ENABLE_R:
         try:
@@ -1665,6 +1676,108 @@ def metrics_keys():
     dic = get_metrics(X,y)
     return list(dic[0].keys())
 
+
+# Defines how each metric's polarity, i.e., how to compare each metric.  
+#   np.argmax - the metric and classifiability are positively correlated
+#   np.argmin - the metric and classifiability are negatively correlated
+metric_polarity_dict={
+    'classification.ACC': np.argmax,
+    'classification.Kappa': np.argmax,
+    'classification.F1_Score': np.argmax ,
+    'classification.Jaccard': np.argmin,
+    'classification.Precision': np.argmax,
+    'classification.Recall': np.argmax,
+    'classification.McNemar': np.argmin,
+    'classification.McNemar.CHI2':np.argmax,
+    'classification.CochranQ': np.argmin,
+    'classification.CochranQ.T':np.argmax,
+    'classification.CrossEntropy':np.argmin,
+    'classification.Mean_KLD': np.argmin,
+    'classification.AP': np.argmax,
+    'classification.Brier':np.argmin ,
+    'classification.ROC_AUC':np.argmax,
+    'classification.PR_AUC':np.argmin,
+    'classification.BER':np.argmin,
+    'classification.SVM.Margin':np.argmax,
+    'correlation.IG':np.argmax,
+    'correlation.IG.max':np.argmax,
+    'correlation.r':np.argmax,
+    'correlation.r2':np.argmax,
+    'correlation.r.p':np.argmin,
+    'correlation.r.max':np.argmax,
+    'correlation.r.p.min':np.argmin,
+    'correlation.rho':np.argmax,
+    'correlation.rho.p':np.argmin,
+    'correlation.rho.max':np.argmax,
+    'correlation.rho.p.min':np.argmin,
+    'correlation.tau':np.argmin,
+    'correlation.tau.p':np.argmin,
+    'correlation.tau.max':np.argmax,
+    'correlation.tau.p.min':np.argmin,
+    'test.ES':np.argmax,
+    'test.ES.max':np.argmax,
+    'test.student':np.argmin,
+    'test.student.min':np.argmin,
+    'test.student.min.log10':np.argmin,
+    'test.student.T':np.argmax,
+    'test.student.T.max':np.argmin,
+    'test.ANOVA':np.argmin,
+    'test.ANOVA.min':np.argmin,
+    'test.ANOVA.min.log10':np.argmin,
+    'test.ANOVA.F':np.argmax,
+    'test.ANOVA.F.max':np.argmax,
+    'test.MANOVA':np.argmin,
+    'test.MANOVA.log10':np.argmin,
+    'test.MANOVA.F':np.argmax,
+    'test.MWW':np.argmin,
+    'test.MWW.min':np.argmin,
+    'test.MWW.min.log10':np.argmin,
+    'test.MWW.U':np.argmin,
+    'test.MWW.U.min':np.argmin,
+    'test.KS':np.argmin,
+    'test.KS.min':np.argmin,
+    'test.KS.min.log10':np.argmin,
+    'test.KS.D':np.argmax,
+    'test.KS.D.max':np.argmax,
+    'test.CHISQ':np.argmin,
+    'test.CHISQ.min':np.argmin,
+    'test.CHISQ.min.log10':np.argmin,
+    'test.CHISQ.CHI2':np.argmax,
+    'test.CHISQ.CHI2.max':np.argmax,
+    'test.KW':np.argmin,
+    'test.KW.min':np.argmin,
+    'test.KW.min.log10':np.argmin,
+    'test.KW.H':np.argmax,
+    'test.KW.H.max':np.argmax,
+    'test.Median':np.argmin,
+    'test.Median.min':np.argmin,
+    'test.Median.min.log10':np.argmin,
+    'test.Median.CHI2':np.argmax,
+    'test.Median.CHI2.max':np.argmax,
+    'overlapping.F1.mean':np.argmax,
+    'overlapping.F1.sd':np.argmax,
+    'overlapping.F1v.mean':np.argmax,
+    'overlapping.F1v.sd':np.argmin,
+    'overlapping.F2.mean':np.argmin,
+    'overlapping.F2.sd':np.argmin,
+    'overlapping.F3.mean':np.argmax,
+    'overlapping.F3.sd':np.argmin,
+    'overlapping.F4.mean':np.argmax,
+    'overlapping.F4.sd':np.argmin,
+    'neighborhood.N1':np.argmin,
+    'neighborhood.N2.mean':np.argmin,
+    'neighborhood.N2.sd':np.argmin,
+    'neighborhood.N3.mean':np.argmin,
+    'neighborhood.N3.sd':np.argmax,
+    'neighborhood.N4.mean':np.argmin,
+    'neighborhood.N4.sd':np.argmin,
+    'neighborhood.T1.mean':np.argmax,
+    'neighborhood.T1.sd':np.argmin,
+    'neighborhood.LSC':np.argmax,
+    'linearity.L1.mean':np.argmin,
+    'linearity.L1.sd':np.argmin
+ }
+
 def get_json(X,y):    
     return json.dumps(get_metrics(X,y))
 
@@ -1910,18 +2023,22 @@ def visualize_corr_matrix(dic, cmap = 'coolwarm', threshold = 0.25):
     plt.show()
     
     plt.figure(figsize = (20, 7))
-    plt.bar(names[1:], np.square(dfM.corr().values[0,1:]), 
-    facecolor="none", edgecolor = "black", 
+    corrs = np.square(dfM.corr().values[0,1:])
+    corrs = np.nan_to_num(corrs)
+    plt.bar(names[1:], corrs, facecolor="none", edgecolor = "black", 
     width = 0.8, label = 'R2 effect size') # ,width = 0.6, hatch='x'
-    plt.plot(names[1:], [threshold] * len(names[1:]), '--', 
-    label = 'R2 threshold = ' + str(threshold))
+    if threshold is not None:
+        plt.plot(names[1:], [threshold] * len(names[1:]), '--', 
+        label = 'R2 threshold = ' + str(threshold))
     plt.title('R2 effect size of the correlation with between-class distance')
     plt.xticks(rotation = 90)
     plt.legend(loc = 'lower right')
     plt.show()
 
-    # print(np.where( np.abs(dfM.corr().values[0,1:])>0.9 ))
-    print('Metrics above the threshold (' + str(threshold) + '): ', np.array(names[1:]) [np.where( np.square(dfM.corr().values[0,1:])>threshold )])
+    if threshold is not None:
+        # print(np.where( np.abs(dfM.corr().values[0,1:])>0.9 ))
+        print('Metrics above the threshold (' + str(threshold) + '): ', 
+            np.array(names[1:]) [np.where( corrs > threshold )])
 
 def extract_PC(dic):
 
